// CuteSphere.jsx
// Copyright (c) 2011 Motion Boutique

function addFacet(comp, solidItem, type, pos, idx)
{
	var layer;
	if (type == "Solid") // solid
	{
		layer = comp.layers.add(solidItem);
	}
	else // shape layer
	{
		layer = comp.layers.addShape();
		
		var shapeName = (type == "RectShape") ? "Rect" : "Ellipse";
				
		var grp = layer.property("ADBE Root Vectors Group");
		var path = grp.addProperty("ADBE Vector Shape - " + shapeName);
		var size = path.property("ADBE Vector " + shapeName + " Size");
		size.setValue([solidItem.width,solidItem.height]);
	
		var fill = grp.addProperty("ADBE Vector Graphic - Fill");
		var fillColor = fill.property("ADBE Vector Fill Color");
		fillColor.setValue(solidItem.mainSource.color);		
	}
	layer.name = "Facet " + (idx+1);
	layer.threeDLayer = true;
	layer.position.setValue(layer.position.value + pos);
	layer.orientation.expression = "lookAt(position,[thisComp.width/2,thisComp.height/2,0]);";
	layer.orientation.setValue(layer.orientation.valueAtTime(0,false));
	layer.orientation.expression = "";
	layer.moveToEnd();
}
	
function createSphere(comp, solidItem, columns, rows, radius)
{
	var drho = Math.PI / columns;
	var dtheta = (2.0 * Math.PI) / columns;
	
	var counter = 0;
	
	for (var j = 1; j < rows; j++) 
	{
		var rho = j * drho;

		for (var i = 0; i < columns; i++) 
		{
			var theta = i * dtheta;

			var x = Math.sin(theta) * Math.sin(rho);
			var y = -Math.cos(rho);
			var z = Math.cos(theta) * Math.sin(rho);

			var pos = [x * radius, y * radius, z * radius];
			
			addFacet(comp, solidItem, "CircleShape", pos, counter)
			
			counter++;
		}
	}
}

function run()
{
	var inputData = prompt("Please enter the sphere parameters: columns, rows, radius", "20,20,500");
	
	if (inputData)
	{
		var err = false;
		inputData = inputData.split(",");
		if (inputData.length < 3)
		{
			err = true;
		}
		else
		{
			for (var i = 0; !err && i < 3; i++)
			{
				err = isNaN(inputData[i]);
			}
			if (!err)
			{
				var columns = Math.round(Math.min(Math.max(inputData[0],3), 512));
				var rows = Math.round(Math.min(Math.max(inputData[1],3), 512));			
				var radius = inputData[2];

				app.beginUndoGroup("Cute Sphere");

				var comp = app.project.items.addComp("Cute_Sphere_" + columns + "-" + rows + "-" + radius, 1280,720,1,10,25);
				var solid = comp.layers.addSolid([1,1,1],"Facet",100,100,1,10);
				var solidItem = solid.source;
				solid.remove();

				createSphere(comp, solidItem, columns, rows, radius);

				app.endUndoGroup();
			}
		}
		
		if (err)
		{
			alert("Input error: please make sure to separate the three parameters with comma (e.g.: 20,20,500");
		}
	}
}

run();